@@ -2,6 +2,8 @@
 Contains many of the most famous sequences of the OEIS as Python Functions
 each function returns a list containing terms of the sequence"""
 
+import math
+
 
 def check(n):
     """checks if arg 'n' is a positive int"""
@@ -13,6 +15,18 @@ def check(n):
     return n
 
 
+def isprime(n):
+    """returns True if n is prime"""
+
+    if n in range(2, 4):
+        return True
+
+    for i in range(2, int(pow(n, 0.5) + 1)):
+        if not (n % i):
+            return False
+    return True
+
+
 def whole(n):
     """returns the first n Whole Numbers
     0, 1, 2, 3, 4, 5, 6, 7, 8, 9, ..., n-1"""
@@ -188,6 +202,24 @@ def stella_octangula(n):
     return [i * ((2 * pow(i, 2)) - 1) for i in whole(n)]
 
 
+def central_polygon(n):
+    """returns the first n terms of the Central Polygon Number Sequence
+    Central Polygon Number:
+        the maximum number of pieces of a circular disc that can be made with
+        a given number of straight cuts
+        Tₙ = (n² + n + 2) / 2"""
+
+    return [(pow(n, 2) + n+2) // 2 for n in whole(n)]
+
+
+def magic_constants(n):
+    """returns the first n Magic Constants
+    Magic Constant:
+        Mₙ = n(n²+1) / 2"""
+
+    return [i * (pow(i, 2)+1) // 2 for i in whole(n)]
+
+
 def woodall(n):
     """returns the first n terms of the Woodall Number Sequence
     Woodall Number:
@@ -215,6 +247,31 @@ def pronic(n):
     return [i * (i+1) for i in whole(n)]
 
 
+def arithmetic(n):
+    """returns the first n terms of the Arithmetic Number Sequence
+    Arithmetic Number:
+        a number such that the average of its positive divisors is also an
+        integer"""
+
+    n, numbers, num = check(n), list(), 1
+
+    def average(x):
+        """returns the average of the positive divisors of x"""
+
+        divisors = list()
+        for i in range(1, x+1):
+            if not (x % i):
+                divisors.append(i)
+        return float(sum(divisors) / len(divisors))
+
+    while len(numbers) < n:
+        if average(num).is_integer():
+            numbers.append(num)
+        num += 1
+
+    return numbers
+
+
 def carol(n):
     """returns the first n terms of the Carol Number Sequence
     Carol Number:
@@ -268,8 +325,7 @@ def pascal(n):
     n-th row of Pascal's Triangle is given by:
         Pₙ = ⁿCᵣ for r ∈ range(n+1)"""
 
-    from math import comb
-    return [comb(n, r) for r in range(n+1)]
+    return [math.comb(n, r) for r in range(n+1)]
 
 
 def gould(n):
@@ -290,6 +346,24 @@ def gould(n):
     return numbers
 
 
+def central_binomial(n):
+    """returns the first n Central Binomial Coefficients
+    Central Binomial Coefficient:
+        the coefficient that shows up exactly in the middle of the even numbered
+        rows of Pascal's triangle"""
+
+    return [math.comb(2*i, i) for i in whole(n)]
+
+
+def catalan(n):
+    """returns the first n terms of the Catalan Number Sequence
+    Catalan Number:
+        number of the form (2n!) / (n!(n+1)!)
+        Cₙ = ²ⁿCₙ / (n+1)"""
+
+    return [math.comb(2*i, i) // (i+1) for i in whole(n)]
+
+
 def van_eck(n):
     """returns first n terms of the Van-Eck Sequence
     Van-Eck Sequence is defined as:
@@ -365,6 +439,57 @@ def look_say(n):
     return numbers
 
 
+def aronson(n):
+    """returns the first n terms of the Aronson Sequence
+    Aronson Sequence is defined as:
+        the index of English Letter "T" or "t" in the sentence
+        "T is the first, fourth, eleventh, ... letter in this sentence"
+        ignoring spaces and punctuation marks"""
+
+    sentence = "_tisthefirstfourth"
+    n, numbers = check(n), list()
+
+    def name(number):
+        """returns the name of an integer"""
+
+        names = {
+            0: "zero", 1: "first", 2: "second", 3: "third", 4: "fourth",
+            5: "fifth", 6: "sixth", 7: "seventh", 8: "eighth", 9: "ninth",
+            11: "eleventh", 12: "twelfth", 13: "thirteenth", 14: "fourteenth",
+            15: "fifteenth", 16: "sixteenth", 17: "seventeenth", 10: "ten",
+            18: "eighteenth", 19: "nineteenth", 30: "thirty", 40: "forty",
+            20: "twenty",  50: "fifty", 60: "sixty", 70: "seventy",
+            80: "eighty", 90: "ninety",
+        }
+        orders = {
+            **dict.fromkeys([3], ("hundred", 10**2)),
+            **dict.fromkeys([4, 5, 6], ("thousand", 10**3)),
+            **dict.fromkeys([7, 8, 9], ("million", 10**6)),
+            **dict.fromkeys([10, 11, 12], ("billion", 10**9)),
+            **dict.fromkeys([13, 14, 15], ("trillion", 10**12)),
+        }
+
+        order = len(str(number))
+
+        if number in names:
+            return names[number]
+        if order == 2:
+            return f"{name(number // 10*10)}{name(number % 10)}"
+        place, div = orders[order]
+        slash, percent = number // div, number % div
+        if not percent:
+            return f"{name(slash)}{place}"
+        return f"{name(slash)}{place}{name(percent)}"
+
+    while len(numbers) < n:
+        i = sentence.index("t")
+        numbers.append(i)
+        sentence = sentence.replace("t", "_", 1)
+        if i > 4:
+            sentence += name(i)
+    return numbers
+
+
 def baum_sweet(n):
     """returns the first n terms of the Baum-Sweet Sequence
     Baum Sweet Sequence is defined as:
@@ -447,6 +572,33 @@ def negatribonacci(n):
     return [pow(-1, i+1) * tribonacci(n)[i] for i in range(n)]
 
 
+def lucas(n):
+    """returns the first n terms of the Lucas Sequence
+    Lucas Sequence is defined as:
+        Lₙ = 2 if n == 0
+        Lₙ = 1 if n == 1
+        Lₙ = Lₙ₋₁ + Lₙ₋₂ otherwise"""
+
+    n, numbers = check(n), [2, 1]
+
+    if n in range(1, 3):
+        return numbers[:n]
+
+    while len(numbers) < n:
+        numbers.append(numbers[-1] + numbers[-2])
+
+    return numbers
+
+
+def negalucas(n):
+    """returns first n terms of the NegaLucas Sequence
+    NegaFibonacci Sequence is defined as:
+        L₋ₙ = (-1)ⁿLₙ
+        where Lₙ is the n-th Lucas Number"""
+
+    return [pow(-1, i) * lucas(n)[i] for i in range(n)]
+
+
 def supergolden(n):
     """returns the first n terms of the Supergolden Sequence
     Supergolden Sequence is defined as:
@@ -553,6 +705,33 @@ def sylvester(n):
     return numbers
 
 
+def euclid_mullin(n):
+    """returns the first n terms of the Euclid-Mullin Sequence
+    Euclid-Mullin Sequence is defined as:
+        Tₙ = 2 if n == 1
+        Tₙ = smallest prime factor of product(euclid_mullin(n-1))+1 otherwise"""
+
+    n, numbers = check(n), [2]
+
+    def smallest_factor(x):
+        """returns the smallest prime factor of x"""
+
+        if isprime(x):
+            return x
+
+        for i in range(2, x+1):
+            if isprime(i) and (not x % i):
+                return i
+
+    while len(numbers) < n:
+        num = 1
+        for i in numbers:
+            num *= i
+        numbers.append(smallest_factor(num + 1))
+
+    return numbers
+
+
 def sophie_germain(n):
     """returns the first n terms of the Sophie-Germain Prime Number Sequence
     Sophie Germain Prime Number:
@@ -560,17 +739,6 @@ def sophie_germain(n):
 
     n, numbers, num = check(n), list(), 2
 
-    def isprime(x):
-        """returns True if x is prime"""
-
-        if x in range(2, 4):
-            return True
-
-        for i in range(2, int(pow(x, 0.5) + 1)):
-            if not (x % i):
-                return False
-        return True
-
     while len(numbers) < n:
         if isprime(num) and isprime(2*num + 1):
             numbers.append(num)
@@ -595,7 +763,7 @@ def circular_prime(n):
             all_x.append(int(x))
         return all_x
 
-    def isprime(x):
+    def is_circ_prime(x):
         """returns True if all cylic permutations of x are prime"""
 
         if x in (2, 3, 5, 7):
@@ -608,7 +776,7 @@ def circular_prime(n):
         return True
 
     while len(numbers) < n:
-        if isprime(num):
+        if is_circ_prime(num):
             for z in perm(num):
                 if z in numbers:
                     break
@@ -619,72 +787,88 @@ def circular_prime(n):
     return numbers
 
 
-def catalan(n):
-    """returns the first n terms of the Catalan Number Sequence
-    Catalan Number:
-        number of the form (2n!) / (n!(n+1)!)
-        Cₙ = ²ⁿCₙ / (n+1)"""
+def prime_powers(n):
+    """returns the first n Prime Power Numbers
+    Prime Power Number:
+        number of the form xⁿ where x is prime and n is any positive integer"""
 
-    from math import comb
-    return [comb(2*i, i) // (i+1) for i in whole(n)]
+    n, numbers, num = check(n), [2], 3
 
+    def is_prime_power(x):
+        """returns True if x is a Prime Power Number"""
 
-def aronson(n):
-    """returns the first n terms of the Aronson Sequence
-    Aronson Sequence is defined as:
-        the index of English Letter "T" or "t" in the sentence
-        "T is the first, fourth, eleventh, ... letter in this sentence"
-        ignoring spaces and punctuation marks"""
-
-    sentence = "_tisthefirstfourth"
-    n, numbers = check(n), list()
-
-    def name(number):
-        """returns the name of an integer"""
-
-        names = {
-            0: "zero", 1: "first", 2: "second", 3: "third", 4: "fourth",
-            5: "fifth", 6: "sixth", 7: "seventh", 8: "eighth", 9: "ninth",
-            11: "eleventh", 12: "twelfth", 13: "thirteenth", 14: "fourteenth",
-            15: "fifteenth", 16: "sixteenth", 17: "seventeenth", 10: "ten",
-            18: "eighteenth", 19: "nineteenth", 30: "thirty", 40: "forty",
-            20: "twenty",  50: "fifty", 60: "sixty", 70: "seventy",
-            80: "eighty", 90: "ninety",
-        }
-        orders = {
-            **dict.fromkeys([3], ("hundred", 10**2)),
-            **dict.fromkeys([4, 5, 6], ("thousand", 10**3)),
-            **dict.fromkeys([7, 8, 9], ("million", 10**6)),
-            **dict.fromkeys([10, 11, 12], ("billion", 10**9)),
-            **dict.fromkeys([13, 14, 15], ("trillion", 10**12)),
-        }
-
-        order = len(str(number))
-
-        if number in names:
-            return names[number]
-        if order == 2:
-            return f"{name(number // 10*10)}{name(number % 10)}"
-        place, div = orders[order]
-        slash, percent = number // div, number % div
-        if not percent:
-            return f"{name(slash)}{place}"
-        return f"{name(slash)}{place}{name(percent)}"
+        copy = x
+        for i in prime(x):
+            while copy != 1:
+                if copy % i:
+                    break
+                copy //= i
+            else:
+                return True
+            copy = x
+        return False
 
     while len(numbers) < n:
-        i = sentence.index("t")
-        numbers.append(i)
-        sentence = sentence.replace("t", "_", 1)
-        if i > 4:
-            sentence += name(i)
+        if is_prime_power(num):
+            numbers.append(num)
+        num += 1
+
     return numbers
 
 
-def central_polygon(n):
-    """returns the first n terms of the Central Polygon Number Sequence
-    Central Polygon Number:
-        the maximum number of pieces of a circular disc that can be made with
-        a given number of straight cuts
-        Tₙ = (n² + n + 2) / 2"""
+def semiprime(n):
+    """returns the first n Semi-Prime Numbers
+    Semi-Prime Number:
+        the product of two prime numbers, not necessarily distinct"""
 
-    return [(pow(n, 2) + n+2) // 2 for n in whole(n)]
\ No newline at end of file
+    n, numbers, num = check(n), list(), 4
+
+    def factors(x):
+        """returns the number of prime factors of x (not unique)"""
+
+        total, i = 0, 2
+        while x != 1:
+            if not isprime(i):
+                i += 1
+            if not (x % i):
+                x //= i
+                total += 1
+            else:
+                i += 1
+        return total
+
+    while len(numbers) < n:
+        if factors(num) == 2:
+            numbers.append(num)
+        num += 1
+
+    return numbers
+
+
+def sphenic(n):
+    """returns the first n terms of the Sphenic Number Sequence
+    Sphenic Number:
+        number that is the product of three distinct primes"""
+
+    n, numbers, num = check(n), list(), 30
+
+    def factors(x):
+        """returns the number of unique prime factors of x"""
+
+        divs, i = list(), 2
+        while x != 1:
+            if not isprime(i):
+                i += 1
+            if not (x % i):
+                x //= i
+                divs.append(i)
+            else:
+                i += 1
+        return divs
+
+    while len(numbers) < n:
+        if len(f := factors(num)) == len(set(f)) == 3:
+            numbers.append(num)
+        num += 1
+
+    return numbers
\ No newline at end of file
